@@ -29,18 +29,22 @@
 #include <stdio.h>
 
 #include <glib.h>
+#include <ell/ell.h>
 
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/netreg.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
 #include "common.h"
-#include "atmodem.h"
 #include "vendor.h"
 
+#include "network-registration.h"
+
 static const char *none_prefix[] = { NULL };
 static const char *creg_prefix[] = { "+CREG:", NULL };
 static const char *cops_prefix[] = { "+COPS:", NULL };
@@ -51,20 +55,6 @@ static const char *smoni_prefix[] = { "^SMONI:", NULL };
 static const char *zpas_prefix[] = { "+ZPAS:", NULL };
 static const char *option_tech_prefix[] = { "_OCTI:", "_OUWCTI:", NULL };
 
-struct netreg_data {
-	GAtChat *chat;
-	char mcc[OFONO_MAX_MCC_LENGTH + 1];
-	char mnc[OFONO_MAX_MNC_LENGTH + 1];
-	int signal_index; /* If strength is reported via CIND */
-	int signal_min; /* min strength reported via CIND */
-	int signal_max; /* max strength reported via CIND */
-	int signal_invalid; /* invalid strength reported via CIND */
-	int tech;
-	struct ofono_network_time time;
-	guint nitz_timeout;
-	unsigned int vendor;
-};
-
 struct tech_query {
 	int status;
 	int lac;
@@ -209,7 +199,7 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_netreg_status_cb_t cb = cbd->cb;
 	int status, lac, ci, tech;
 	struct ofono_error error;
-	struct netreg_data *nd = cbd->user;
+	struct at_netreg_data *nd = cbd->user;
 
 	decode_at_error(&error, g_at_result_final_response(result));
 
@@ -250,7 +240,7 @@ static void zte_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	struct ofono_netreg *netreg = cbd->data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (ok)
 		nd->tech = zte_parse_tech(result);
@@ -262,7 +252,7 @@ static void option_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	struct ofono_netreg *netreg = cbd->data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (ok)
 		nd->tech = option_parse_tech(result);
@@ -270,11 +260,11 @@ static void option_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		nd->tech = -1;
 }
 
-static void at_registration_status(struct ofono_netreg *netreg,
+void at_registration_status(struct ofono_netreg *netreg,
 					ofono_netreg_status_cb_t cb,
 					void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	cbd->user = nd;
@@ -337,7 +327,7 @@ static void at_registration_status(struct ofono_netreg *netreg,
 static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(cbd->user);
+	struct at_netreg_data *nd = ofono_netreg_get_data(cbd->user);
 	ofono_netreg_operator_cb_t cb = cbd->cb;
 	struct ofono_network_operator op;
 	GAtResultIter iter;
@@ -369,14 +359,9 @@ static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (g_at_result_iter_next_number(&iter, &tech) == FALSE)
 		tech = ACCESS_TECHNOLOGY_GSM;
 
-	strncpy(op.name, name, OFONO_MAX_OPERATOR_NAME_LENGTH);
-	op.name[OFONO_MAX_OPERATOR_NAME_LENGTH] = '\0';
-
-	strncpy(op.mcc, nd->mcc, OFONO_MAX_MCC_LENGTH);
-	op.mcc[OFONO_MAX_MCC_LENGTH] = '\0';
-
-	strncpy(op.mnc, nd->mnc, OFONO_MAX_MNC_LENGTH);
-	op.mnc[OFONO_MAX_MNC_LENGTH] = '\0';
+	l_strlcpy(op.name, name, sizeof(op.name));
+	l_strlcpy(op.mcc, nd->mcc, sizeof(op.mcc));
+	l_strlcpy(op.mnc, nd->mnc, sizeof(op.mnc));
 
 	/* Set to current */
 	op.status = 2;
@@ -398,7 +383,7 @@ error:
 static void cops_numeric_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(cbd->user);
+	struct at_netreg_data *nd = ofono_netreg_get_data(cbd->user);
 	ofono_netreg_operator_cb_t cb = cbd->cb;
 	GAtResultIter iter;
 	const char *str;
@@ -450,10 +435,10 @@ error:
 	g_free(cbd);
 }
 
-static void at_current_operator(struct ofono_netreg *netreg,
+void at_current_operator(struct ofono_netreg *netreg,
 				ofono_netreg_operator_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
@@ -589,10 +574,10 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	g_free(list);
 }
 
-static void at_list_operators(struct ofono_netreg *netreg,
+void at_list_operators(struct ofono_netreg *netreg,
 				ofono_netreg_operator_list_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=?", cops_prefix,
@@ -615,10 +600,10 @@ static void register_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, cbd->data);
 }
 
-static void at_register_auto(struct ofono_netreg *netreg,
+void at_register_auto(struct ofono_netreg *netreg,
 				ofono_netreg_register_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=0", none_prefix,
@@ -630,11 +615,11 @@ static void at_register_auto(struct ofono_netreg *netreg,
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
-static void at_register_manual(struct ofono_netreg *netreg,
+void at_register_manual(struct ofono_netreg *netreg,
 				const char *mcc, const char *mnc,
 				ofono_netreg_register_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
@@ -723,7 +708,7 @@ static void ifx_xhomezr_notify(GAtResult *result, gpointer user_data)
 static void ifx_xreg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int state;
 	const char *band;
 	GAtResultIter iter;
@@ -792,15 +777,26 @@ static void ifx_xciev_notify(GAtResult *result, gpointer user_data)
 	 */
 }
 
-static void ifx_xcsq_notify(GAtResult *result, gpointer user_data)
+static void ifx_quec_csq_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int rssi, ber, strength;
 	GAtResultIter iter;
+	const char *prefix;
 
 	g_at_result_iter_init(&iter, result);
 
-	if (!g_at_result_iter_next(&iter, "+XCSQ:"))
+	switch (nd->vendor) {
+	case  OFONO_VENDOR_QUECTEL_SERIAL:
+		prefix = "+CSQN:";
+		break;
+	default:
+		prefix = "+XCSQ:";
+		break;
+	}
+
+	if (!g_at_result_iter_next(&iter, prefix))
 		return;
 
 	if (!g_at_result_iter_next_number(&iter, &rssi))
@@ -822,7 +818,7 @@ static void ifx_xcsq_notify(GAtResult *result, gpointer user_data)
 static void ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int strength, ind;
 	GAtResultIter iter;
 
@@ -851,7 +847,7 @@ static void ciev_notify(GAtResult *result, gpointer user_data)
 static void telit_ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *signal_identifier = "rssi";
 	const char *ind_str;
 	int strength;
@@ -882,7 +878,7 @@ static void telit_ciev_notify(GAtResult *result, gpointer user_data)
 static void gemalto_ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *signal_identifier = "rssi";
 	const char *ind_str;
 	int strength;
@@ -915,7 +911,7 @@ static void gemalto_ciev_notify(GAtResult *result, gpointer user_data)
 static void ctzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *tz;
 	GAtResultIter iter;
 
@@ -937,7 +933,7 @@ static void ctzv_notify(GAtResult *result, gpointer user_data)
 static void tlts_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	char tz[4];
 	const char *time;
@@ -969,10 +965,70 @@ static void tlts_notify(GAtResult *result, gpointer user_data)
 	ofono_netreg_time_notify(netreg, &nd->time);
 }
 
+static void quectel_qind_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
+	int rssi, ber, strength;
+	const char *str;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+QIND:"))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &str))
+		return;
+
+	if (g_str_equal("csq", str)) {
+		if (!g_at_result_iter_next_number(&iter, &rssi))
+			return;
+
+		if (!g_at_result_iter_next_number(&iter, &ber))
+			return;
+
+		DBG("rssi %d ber %d", rssi, ber);
+
+		if ((rssi == 99) || (rssi == 199))
+			strength = -1;
+		else if (rssi > 99) {
+			rssi -= 100;
+			strength = (rssi * 100) / 91;
+		} else
+			strength = (rssi * 100) / 31;
+
+		ofono_netreg_strength_notify(netreg, strength);
+		return;
+	}
+
+	if (g_str_equal("act", str)) {
+		nd->tech = -1;
+		if (!g_at_result_iter_next_string(&iter, &str))
+			return;
+
+		DBG("technology %s", str);
+		if (g_str_equal("GSM", str))
+			nd->tech = ACCESS_TECHNOLOGY_GSM;
+		else if (g_str_equal("EGPRS", str))
+			nd->tech = ACCESS_TECHNOLOGY_GSM_EGPRS;
+		else if (g_str_equal("WCDMA", str))
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN;
+		else if (g_str_equal("HSDPA", str))
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA;
+		else if (g_str_equal("HSUPA", str))
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSUPA;
+		else if (g_str_equal("HSDPA&HSUPA", str))
+			nd->tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA_HSUPA;
+		else if (g_str_equal("LTE", str))
+			nd->tech = ACCESS_TECHNOLOGY_EUTRAN;
+	}
+}
+
 static gboolean notify_time(gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	nd->nitz_timeout = 0;
 
@@ -984,7 +1040,7 @@ static gboolean notify_time(gpointer user_data)
 static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	const char *tz, *time;
 	GAtResultIter iter;
@@ -1022,7 +1078,7 @@ static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
 static void ifx_ctzdst_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int dst;
 	GAtResultIter iter;
 
@@ -1050,7 +1106,7 @@ static void cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_netreg_strength_cb_t cb = cbd->cb;
-	struct netreg_data *nd = cbd->user;
+	struct at_netreg_data *nd = cbd->user;
 	int index;
 	int strength;
 	GAtResultIter iter;
@@ -1104,7 +1160,7 @@ static void huawei_rssi_notify(GAtResult *result, gpointer user_data)
 static void huawei_mode_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	int mode, submode;
 
@@ -1132,7 +1188,7 @@ static void huawei_mode_notify(GAtResult *result, gpointer user_data)
 static void huawei_hcsq_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *mode;
 
@@ -1153,7 +1209,7 @@ static void huawei_hcsq_notify(GAtResult *result, gpointer user_data)
 static void huawei_nwtime_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	char tz[4];
 	const char *date, *time, *dst;
@@ -1228,10 +1284,10 @@ static void csq_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, strength, cbd->data);
 }
 
-static void at_signal_strength(struct ofono_netreg *netreg,
+void at_signal_strength(struct ofono_netreg *netreg,
 				ofono_netreg_strength_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	cbd->user = nd;
@@ -1258,7 +1314,7 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 static void mbm_etzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	const char *tz, *time, *timestamp;
 	GAtResultIter iter;
@@ -1307,7 +1363,7 @@ static void mbm_etzv_notify(GAtResult *result, gpointer user_data)
 static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	int mode, gsm, umts;
 
@@ -1359,7 +1415,7 @@ static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 static void icera_nwstate_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *mccmnc, *tech, *state;
 	int rssi;
@@ -1427,7 +1483,7 @@ static int cnti_to_tech(const char *cnti)
 static void gobi_cnti_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *tech;
 	int option;
@@ -1452,7 +1508,7 @@ static void gobi_cnti_notify(GAtResult *result, gpointer user_data)
 static void nw_cnti_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *tech;
 	int option;
@@ -1478,7 +1534,7 @@ static void cnti_query_tech_cb(gboolean ok, GAtResult *result,
 						gpointer user_data)
 {
 	struct tech_query *tq = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(tq->netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(tq->netreg);
 
 	ofono_netreg_status_notify(tq->netreg,
 			tq->status, tq->lac, tq->ci, nd->tech);
@@ -1518,7 +1574,7 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	int status, lac, ci, tech;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct tech_query *tq;
 
 	if (at_util_parse_reg_unsolicited(result, "+CREG:", &status,
@@ -1587,7 +1643,7 @@ static void at_cmer_not_supported(struct ofono_netreg *netreg)
 static void at_cmer_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (!ok) {
 		at_cmer_not_supported(netreg);
@@ -1646,7 +1702,7 @@ static inline ofono_bool_t append_cmer_element(char *buf, int *len, int cap,
 }
 
 static ofono_bool_t build_cmer_string(char *buf, int *cmer_opts,
-					struct netreg_data *nd)
+					struct at_netreg_data *nd)
 {
 	const char *ind;
 	int len = sprintf(buf, "AT+CMER=");
@@ -1715,7 +1771,7 @@ static void at_cmer_query_cb(ofono_bool_t ok, GAtResult *result,
 				gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	int cmer_opts_cnt = 5; /* See 27.007 Section 8.10 */
 	int cmer_opts[cmer_opts_cnt];
@@ -1763,7 +1819,7 @@ error:
 static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *str;
 	char *signal_identifier = "signal";
@@ -1870,7 +1926,7 @@ error:
 static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (!ok) {
 		ofono_error("Unable to initialize Network Registration");
@@ -1979,7 +2035,7 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		/* Register for specific signal strength reports */
 		g_at_chat_register(nd->chat, "+XCIEV:", ifx_xciev_notify,
 						FALSE, netreg, NULL);
-		g_at_chat_register(nd->chat, "+XCSQ:", ifx_xcsq_notify,
+		g_at_chat_register(nd->chat, "+XCSQ:", ifx_quec_csq_notify,
 						FALSE, netreg, NULL);
 		g_at_chat_send(nd->chat, "AT+XCSQ=1", none_prefix,
 						NULL, NULL, NULL);
@@ -2059,6 +2115,24 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	case OFONO_VENDOR_SAMSUNG:
 		/* Signal strength reporting via CIND is not supported */
 		break;
+	case OFONO_VENDOR_QUECTEL_EC2X:
+		g_at_chat_register(nd->chat, "+QIND:",
+				quectel_qind_notify, FALSE, netreg, NULL);
+		/* Register for specific signal strength reports */
+		g_at_chat_send(nd->chat, "AT+QINDCFG=\"csq\",1", none_prefix,
+				NULL, NULL, NULL);
+
+		/* Register for network technology updates */
+		g_at_chat_send(nd->chat, "AT+QINDCFG=\"act\",1", none_prefix,
+				NULL, NULL, NULL);
+		break;
+	case OFONO_VENDOR_QUECTEL_SERIAL:
+		g_at_chat_register(nd->chat, "+CSQN:",
+				ifx_quec_csq_notify, FALSE, netreg, NULL);
+		/* Register for specific signal strength reports */
+		g_at_chat_send(nd->chat, "AT+QEXTUNSOL=\"SQ\",1", none_prefix,
+				NULL, NULL, NULL);
+		break;
 	default:
 		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
 				cind_support_cb, netreg, NULL);
@@ -2073,7 +2147,7 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 static void at_creg_test_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	gint range[2];
 	GAtResultIter iter;
 	int creg1 = 0;
@@ -2121,9 +2195,9 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 				void *data)
 {
 	GAtChat *chat = data;
-	struct netreg_data *nd;
+	struct at_netreg_data *nd;
 
-	nd = g_new0(struct netreg_data, 1);
+	nd = g_new0(struct at_netreg_data, 1);
 
 	nd->chat = g_at_chat_clone(chat);
 	nd->vendor = vendor;
@@ -2144,9 +2218,9 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 	return 0;
 }
 
-static void at_netreg_remove(struct ofono_netreg *netreg)
+void at_netreg_remove(struct ofono_netreg *netreg)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (nd->nitz_timeout)
 		g_source_remove(nd->nitz_timeout);
@@ -2158,7 +2232,6 @@ static void at_netreg_remove(struct ofono_netreg *netreg)
 }
 
 static const struct ofono_netreg_driver driver = {
-	.name				= "atmodem",
 	.probe				= at_netreg_probe,
 	.remove				= at_netreg_remove,
 	.registration_status		= at_registration_status,
@@ -2169,12 +2242,4 @@ static const struct ofono_netreg_driver driver = {
 	.strength			= at_signal_strength,
 };
 
-void at_netreg_init(void)
-{
-	ofono_netreg_driver_register(&driver);
-}
-
-void at_netreg_exit(void)
-{
-	ofono_netreg_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(netreg, atmodem, &driver)
