@@ -864,6 +864,7 @@ static void phonesim_post_sim(struct ofono_modem *modem)
 
 	ofono_radio_settings_create(modem, 0, "phonesim", data->chat);
 	ofono_sim_auth_create(modem);
+	ofono_lte_create(modem, 0, "atmodem", data->chat);
 }
 
 static void phonesim_post_online(struct ofono_modem *modem)
@@ -910,7 +911,6 @@ static void phonesim_post_online(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver phonesim_driver = {
-	.name		= "phonesim",
 	.probe		= phonesim_probe,
 	.remove		= phonesim_remove,
 	.enable		= phonesim_enable,
@@ -921,6 +921,8 @@ static struct ofono_modem_driver phonesim_driver = {
 	.post_online	= phonesim_post_online,
 };
 
+OFONO_MODEM_DRIVER_BUILTIN(phonesim, &phonesim_driver)
+
 static int localhfp_probe(struct ofono_modem *modem)
 {
 	struct hfp_slc_info *info;
@@ -1035,7 +1037,6 @@ static void localhfp_pre_sim(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver localhfp_driver = {
-	.name		= "localhfp",
 	.probe		= localhfp_probe,
 	.remove		= localhfp_remove,
 	.enable		= localhfp_enable,
@@ -1043,6 +1044,8 @@ static struct ofono_modem_driver localhfp_driver = {
 	.pre_sim	= localhfp_pre_sim,
 };
 
+OFONO_MODEM_DRIVER_BUILTIN(localhfp, &localhfp_driver)
+
 static struct ofono_modem *create_modem(GKeyFile *keyfile, const char *group)
 {
 	const char *driver = "phonesim";
@@ -1143,15 +1146,8 @@ done:
 
 static int phonesim_init(void)
 {
-	int err;
 	char *conf_override = getenv("OFONO_PHONESIM_CONFIG");
 
-	err = ofono_modem_driver_register(&phonesim_driver);
-	if (err < 0)
-		return err;
-
-	ofono_modem_driver_register(&localhfp_driver);
-
 	if (conf_override)
 		parse_config(conf_override);
 	else
@@ -1172,8 +1168,6 @@ static void phonesim_exit(void)
 
 	g_slist_free(modem_list);
 	modem_list = NULL;
-
-	ofono_modem_driver_unregister(&phonesim_driver);
 }
 
 OFONO_PLUGIN_DEFINE(phonesim, "Phone Simulator driver", VERSION,
