@@ -309,14 +309,12 @@ static gboolean sco_connect_cb(GIOChannel *io, GIOCondition cond,
 
 {
 	struct ofono_handsfree_card *card = user_data;
-	DBusMessage *reply;
+	DBusMessage *reply = NULL;
 	int sk;
 
-	if (agent == NULL) {
-		/* There's no agent, so there's no one to reply to */
-		reply = NULL;
+	/* There's no agent, so there's no one to reply to */
+	if (agent == NULL)
 		goto done;
-	}
 
 	if (cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)) {
 		if (card->msg)
@@ -326,6 +324,13 @@ static gboolean sco_connect_cb(GIOChannel *io, GIOCondition cond,
 
 	sk = g_io_channel_unix_get_fd(io);
 
+	if (card->msg && dbus_message_has_member(card->msg, "Acquire")) {
+		reply = g_dbus_create_reply(card->msg, DBUS_TYPE_UNIX_FD, &sk,
+					DBUS_TYPE_BYTE, &card->selected_codec,
+					DBUS_TYPE_INVALID);
+		goto done;
+	}
+
 	send_new_connection(card->path, sk, card->selected_codec);
 
 	close(sk);
@@ -403,6 +408,9 @@ static const GDBusMethodTable card_methods[] = {
 			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
 			card_get_properties) },
 	{ GDBUS_ASYNC_METHOD("Connect", NULL, NULL, card_connect) },
+	{ GDBUS_ASYNC_METHOD("Acquire", NULL,
+			GDBUS_ARGS({"sco", "h"}, {"codec", "y"}),
+			card_connect) },
 	{ }
 };
 
@@ -457,10 +465,8 @@ void *ofono_handsfree_card_get_data(struct ofono_handsfree_card *card)
 void ofono_handsfree_card_set_remote(struct ofono_handsfree_card *card,
 					const char *remote)
 {
-	if (card->remote)
-		g_free(card->remote);
-
-	card->remote = g_strdup(remote);
+	l_free(card->remote);
+	card->remote = l_strdup(remote);
 }
 
 const char *ofono_handsfree_card_get_remote(struct ofono_handsfree_card *card)
@@ -471,10 +477,8 @@ const char *ofono_handsfree_card_get_remote(struct ofono_handsfree_card *card)
 void ofono_handsfree_card_set_local(struct ofono_handsfree_card *card,
 					const char *local)
 {
-	if (card->local)
-		g_free(card->local);
-
-	card->local = g_strdup(local);
+	l_free(card->local);
+	card->local = l_strdup(local);
 }
 
 const char *ofono_handsfree_card_get_local(struct ofono_handsfree_card *card)
@@ -573,7 +577,7 @@ int ofono_handsfree_card_register(struct ofono_handsfree_card *card)
 
 	next_card_id += 1;
 
-	card->path = g_strdup(path);
+	card->path = l_strdup(path);
 	emit_card_added(card);
 
 	return 0;
@@ -598,7 +602,7 @@ static void card_unregister(struct ofono_handsfree_card *card)
 
 	emit_card_removed(card);
 
-	g_free(card->path);
+	l_free(card->path);
 	card->path = NULL;
 }
 
@@ -614,8 +618,8 @@ void ofono_handsfree_card_remove(struct ofono_handsfree_card *card)
 
 	card_list = g_slist_remove(card_list, card);
 
-	g_free(card->remote);
-	g_free(card->local);
+	l_free(card->remote);
+	l_free(card->local);
 
 	if (card->driver && card->driver->remove)
 		card->driver->remove(card);
@@ -655,8 +659,8 @@ static void agent_free(struct agent *agent)
 	if (agent->watch > 0)
 		g_dbus_remove_watch(ofono_dbus_get_connection(), agent->watch);
 
-	g_free(agent->owner);
-	g_free(agent->path);
+	l_free(agent->owner);
+	l_free(agent->path);
 	g_free(agent);
 }
 
@@ -781,8 +785,8 @@ static DBusMessage *am_agent_register(DBusConnection *conn,
 	}
 
 	agent = g_new0(struct agent, 1);
-	agent->owner = g_strdup(sender);
-	agent->path = g_strdup(path);
+	agent->owner = l_strdup(sender);
+	agent->path = l_strdup(path);
 	agent->watch = g_dbus_add_disconnect_watch(conn, sender,
 						agent_disconnect, NULL, NULL);
 
