@@ -210,19 +210,19 @@ int main(int argc, char **argv)
 
 	if (g_option_context_parse(context, &argc, &argv, &err) == FALSE) {
 		if (err != NULL) {
-			g_printerr("%s\n", err->message);
+			fprintf(stderr, "%s\n", err->message);
 			g_error_free(err);
 			return 1;
 		}
 
-		g_printerr("An unknown error occurred\n");
+		fprintf(stderr, "An unknown error occurred\n");
 		return 1;
 	}
 
 	g_option_context_free(context);
 
 	if (option_version == TRUE) {
-		printf("%s\n", VERSION);
+		fprintf(stdout, "%s\n", VERSION);
 		exit(0);
 	}
 
@@ -236,6 +236,7 @@ int main(int argc, char **argv)
 	event_loop = g_main_loop_new(NULL, FALSE);
 
 	l_log_set_stderr();
+	l_debug("");
 	l_debug_enable("*");
 	l_main_init();
 
@@ -273,12 +274,10 @@ int main(int argc, char **argv)
 
 	__ofono_dbus_init(conn);
 
-	__ofono_modemwatch_init();
-
-	__ofono_manager_init();
+	if (__ofono_modules_init() < 0)
+		goto fail_module_init;
 
 	__ofono_plugin_init(option_plugin, option_noplugin);
-
 	g_free(option_plugin);
 	g_free(option_noplugin);
 
@@ -286,10 +285,9 @@ int main(int argc, char **argv)
 
 	__ofono_plugin_cleanup();
 
-	__ofono_manager_cleanup();
-
-	__ofono_modemwatch_cleanup();
+	__ofono_modules_cleanup();
 
+fail_module_init:
 	__ofono_dbus_cleanup();
 	dbus_connection_unref(conn);
 
